/*
** File: buffer_desc.c  
** Project: VxWorks Driver Framework
** Purpose: Implements buffer description functions.
**
** (C) Copyright Alpha Data 2010
*/

#include "df.h"

DfDescriptionResult
dfBufferDescriptionCreateUser(
  const void* pUserAddress,
  size_t length,
  DfBufferDescription** ppDescription)
{
  DfBufferDescription* pDescription;

  if (0 == length) {
    /* Zero-length buffer is not permitted */
    return DfDescriptionInvalidBuffer;
  }
  if (NULL == pUserAddress) {
    /* Trap NULL pointer */
    return DfDescriptionNullPointer;
  }
  if ((uintptr_t)pUserAddress + length < (uintptr_t)pUserAddress) {
    /* Region wraps back to 0 - not permitted */
    return DfDescriptionInvalidBuffer;
  }

  pDescription = (DfBufferDescription*)dfMalloc(sizeof(*pDescription));
  if (NULL == pDescription) {
    return DfDescriptionNoMemory;
  }
  pDescription->pAddress = pUserAddress;
  pDescription->length = length;

  *ppDescription = pDescription;
  return DfDescriptionSuccess;
}

void
dfBufferDescriptionDelete(
  DfBufferDescription* pDescription)
{
  dfAssert(!pDescription->system.bLocked);
  dfFree(pDescription);
}

boolean_t
dfBufferDescriptionLock(
  DfBufferDescription* pDescription)
{
  /* This is a no operation in VxWorks, assuming valid address */
  dfAssert(!pDescription->system.bLocked);
  pDescription->system.bLocked = TRUE;
  return TRUE;
}

void
dfBufferDescriptionUnlock(
  DfBufferDescription* pDescription)
{
  /* This is a no operation in VxWorks */
  dfAssert(pDescription->system.bLocked);
  pDescription->system.bLocked = FALSE;
}
